package org.one.stone.soup.mjdb.data.field;

/*
 * MJDB (Micro Java Database) Version 1.0
 *
 * Copyright 2000-2001 by Wet-Wired.com Ltd.,
 * Portsmouth England
 * This software is OSI Certified Open Source Software
 * This software is covered by an OSI approved open source licence
 * which can be found at http://www.wet-wired.com/licence/license-os.html
 */

/**
	* The StringField class is a wrapper for a single String value.
	* @author Nik Cross
*/

public class StringField extends Field {
	public static String DEFAULT_ID = "_String";

	private StringBuffer data = new StringBuffer("");
public StringField() {
	super();
}
public StringField(String data) {
	super();

	this.data=new StringBuffer(data);
}
public void append(char c)
{
	data.append(c);
	setChanged();
}
public void append(String s)
{
	data.append(s);
	setChanged();
}
public int compare(String targetStr) {

	String subject=data.toString().toUpperCase();
	String target=targetStr.toUpperCase();

	int subjectSize = subject.length();
	int targetSize = target.length();

	//System.out.println("Compare "+name);

	for(int loop=0;loop<subjectSize;loop++)
	{
		//System.out.println("["+name.charAt(loop)+"] = ["+target.charAt(loop)+"] ?");

		if(loop>=targetSize)
		{
			//System.out.println(name+" longer than "+target);
			return 1;
		}

		if(subject.charAt(loop)==target.charAt(loop))
		{
			continue;
		}
		else
		{
			if(subject.charAt(loop)>target.charAt(loop))
			{
				//System.out.println(subject+" greater than "+target);
				return Field.GT;
			}
			else
			{
				//System.out.println(subject+" less than "+target);
				return Field.LT;
			}
		}
	}

	if(targetSize>subjectSize)
		return Field.LT;
	//System.out.println(subject+" = "+target);

	return Field.EQ;
}
public int compare(Field target) throws CompareException {

	if(target instanceof StringField)
	{
		String term = ((StringField)target).getValue();

		if(term.length()==0)
			return LT;

		if(term.charAt(term.length()-1)=='*')
		{
			return find(term.substring(0,term.length()-1));
		}
		if(term.charAt(term.length()-1)=='!')
		{
			int result = find(term.substring(0,term.length()-1));
			if(result==EQ)
				result = GT;
			else
				result = EQ;

			return result;
		}
		else
			return compare(term);
	}

	throw new CompareException("Can't compare "+target+" with string");

}
public int find(String targetStr) {

	String subject=data.toString().toUpperCase();
	String target=targetStr.toUpperCase();

	if(subject.indexOf(target)>-1)
		return EQ;
	else
		return LT;
}
public String getValue() {
	return data.toString();
}
public String getValue(String name) {
	return data.toString();
}
public void setValue(String newData,Object source) throws LockException {

	if(locked && !isLockOwner(source))
		throw new LockException(this+"is locked");

	if(data.toString().equals(newData))
		return;

	data = new StringBuffer(newData);

	setChanged();

	if(report)
		reportChange(source);
}
public void setValue(String name, String value, Object source) throws LockException
{
	setValue( value,source );
}
public String toString() {
	return ""+data;
}
}
