package org.one.stone.soup.remote.control.client;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.GraphicsDevice;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;

import org.one.stone.soup.grfx.ImageFactory;
import org.one.stone.soup.screen.recorder.ScreenPlayerListener;

/**
 * @author Nik Cross
 *
 */
public class RemoteControlClientFrame extends JFrame implements
		RemoteControlClient
		{

	private JScrollPane scrollPane;
	private JFrame tempFrame;
	private JLabel view;
	private ImageIcon viewIcon;
	private RemoteClientController parent;

	private class Closer extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
			parent.actionPerformed( new ActionEvent(this,0,"disconnect") );
		}
	}
	
	public RemoteControlClientFrame(RemoteClientController parent)
	{
		super("Remote View");
		this.parent = parent;
		
		viewIcon = new ImageIcon( ImageFactory.loadImage("jar://resources/wait.png") );
		view = new JLabel(viewIcon);
		scrollPane = new JScrollPane(view);
		this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		
		getContentPane().add(scrollPane);
		
		this.setSize(100,100);
		addWindowListener( new Closer() );
		
		this.setVisible(true);
	}

	public Component getView()
	{
		return view;
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.remote.control.client.RemoteControlClientInterface#showNewImage(java.awt.Image)
	 */
	public void showNewImage(Image image) {
		if(image==null)
		{
			//this.dispose();
			return;
		}
		
		viewIcon.setImage(image);
		this.repaint(0);
	}

	/* (non-Javadoc)
	 * @see org.one.stone.soup.remote.control.client.RemoteControlClientInterface#playerStopped()
	 */
	public void playerStopped()
	{
	}
	
	public void newFrame()
	{
		parent.requestNextFrame();		
	}
	
	/* (non-Javadoc)
	 * @see org.one.stone.soup.remote.control.client.RemoteControlClientInterface#setFullscreen(boolean)
	 */
	public void setFullscreen(boolean state)
	{
		GraphicsDevice device = this.getGraphicsConfiguration().getDevice();
		if(device.isFullScreenSupported())
		{

			if(state)
			{
				scrollPane.getViewport().remove(view);
				tempFrame = new JFrame();
				tempFrame.setUndecorated(true);
				tempFrame.addKeyListener(parent);
				
				tempFrame.add(view,BorderLayout.CENTER);
				
				device.setFullScreenWindow(tempFrame);
				this.setVisible(true);
			}
			else
			{
				scrollPane.getViewport().setView(view);

				device.setFullScreenWindow(null);
				tempFrame.dispose();
				tempFrame=null;
				
				this.setVisible(true);
			}
		}
	}
}
