package org.one.stone.soup.xapp;

import org.one.stone.soup.xapp.components.form.XForm;
import org.one.stone.soup.xapp.containers.XappFrame;
import org.one.stone.soup.xml.XmlElement;

public abstract class XappPlugin extends XappActionAdapter implements XappPersister,XappSender,XappReceiver{

	public XappFrame frame;
	public XForm form;
	public String formUpdateAction;
	
	public XappPlugin(String pluginFileName,XmlElement definition)
	{
		super(definition);
		
		this.pluginFileName = pluginFileName;
		this.pluginPersistanceFileName = pluginFileName+".data";
	}

	public void buildForm(XmlElement xForm)
	{
		frame =XappRootApplication.getComponentFactory().buildFrame();
		frame.setTitle(getAlias());
		
		form = XappRootApplication.getComponentFactory().buildForm( xForm,new XappComponentStore() );
		frame.setComponent(form);
		
		form.addReceiver( this );
	}
	
	public void buildForm()
	{
		buildForm( getDefinition().getElementByName("form") );
	}
	
	private XappReceiver receiver;
	private String pluginFileName;
	private String pluginPersistanceFileName;
	private XmlElement data;
	
	public String getPluginFileName() {
		return pluginFileName;
	}

	public abstract void initialise();
	
	public String getPersistanceFile()
	{
		return pluginPersistanceFileName;
	}
	
	public void addReceiver(XappReceiver receiver) {
		this.receiver = receiver;
	}

	public void removeReceiver(XappReceiver receiver) {
		this.receiver = null;
	}
	
	public XmlElement getData() {
		return data;
	}

	public void setData(XmlElement data) {
		this.data = data;
	}
	
	public void receiveXappData(XmlElement data) {
		if( data.getName().equals("action") && data.getElementByIndex(0).getName().equals(formUpdateAction) )
		{
			update();
		}
		else if(receiver!=null)
		{
			receiver.receiveXappData(data);
		}
	}

	public void update()
	{
		setData( form.getData() );
	}
	
	public void setPluginFileName(String pluginFileName) {
		this.pluginFileName = pluginFileName;
	}

	public String getFormUpdateAction() {
		return formUpdateAction;
	}

	public void setFormUpdateAction(String formUpdateAction) {
		this.formUpdateAction = formUpdateAction;
	}
}
