package org.one.stone.soup.xapp.dnd;

import java.awt.Component;

public interface XappDropReceiver {

	/**
	 * The drag operation has terminated
	 * with a drop on this <code>DropTarget</code>.
	 * This method is responsible for undertaking
	 * the transfer of the data associated with the
	 * gesture. The <code>DropTargetDropEvent</code>
	 * provides a means to obtain a <code>Transferable</code>
	 * object that represents the data object(s) to
	 * be transfered.<P>
	 * From this method, the <code>DropTargetListener</code>
	 * shall accept or reject the drop via the
	 * acceptDrop(int dropAction) or rejectDrop() methods of the
	 * <code>DropTargetDropEvent</code> parameter.
	 * <P>
	 * Subsequent to acceptDrop(), but not before,
	 * <code>DropTargetDropEvent</code>'s getTransferable()
	 * method may be invoked, and data transfer may be
	 * performed via the returned <code>Transferable</code>'s
	 * getTransferData() method.
	 * <P>
	 * At the completion of a drop, an implementation
	 * of this method is required to signal the success/failure
	 * of the drop by passing an appropriate
	 * <code>boolean</code> to the <code>DropTargetDropEvent</code>'s
	 * dropComplete(boolean success) method.
	 * <P>
	 * Note: The actual processing of the data transfer is not
	 * required to finish before this method returns. It may be
	 * deferred until later.
	 * <P>
	 * @param dtde the <code>DropTargetDropEvent</code>
	 */
	public abstract void drop(java.awt.dnd.DropTargetDropEvent dtde);

	/**
	 * Insert the method's description here.
	 * Creation date: (17/05/03 13:13:02)
	 * @return java.awt.dnd.DropTarget
	 */
	public abstract Component getDropTarget();

	/**
	 * Insert the method's description here.
	 * Creation date: (17/05/03 13:13:02)
	 * @param newDropTarget java.awt.dnd.DropTarget
	 */
	public abstract void setDropTarget(Component target);

}