package org.one.stone.soup.xml.helper;

import java.math.BigDecimal;
import java.util.Hashtable;
import java.util.Vector;

import org.one.stone.soup.stringhelper.StringArrayHelper;
import org.one.stone.soup.stringhelper.StringGenerator;
import org.one.stone.soup.stringhelper.StringParser;
import org.one.stone.soup.xml.XmlDocument;
import org.one.stone.soup.xml.XmlElement;

public class XmlPathSearch {

	private Hashtable processors = new Hashtable();

	private XmlElement root;		// The root document

	private class XmlMatchesFunction extends XmlPathFunction
	{
		private XmlMatchesFunction()
		{
			super("matches");
		}

		public Vector process(XmlElement parent,XmlElement node,String arguments)
		{
			Vector querySet = new Vector();
			
			if(arguments.charAt(0)=='\'')
			{
				arguments = arguments.substring(1,arguments.lastIndexOf('\''));
			}
			
			if(node.getValue()!=null && node.getValue().matches(arguments))
			{
				querySet.addElement(node);
			}
			
			return querySet;
		}
	}
	
	private class XmlPathLastFunction extends XmlPathFunction
	{
		private XmlPathLastFunction()
		{
			super("last");
		}

		public Vector process(XmlElement parent,XmlElement node,String arguments)
		{
			Vector querySet = new Vector();
			querySet.addElement( new Integer(parent.getElementCount()) );
			return querySet;
		}
	}

	private class XmlPathPositionFunction extends XmlPathFunction
	{
		private XmlPathPositionFunction()
		{
			super("position");
		}

		public Vector process(XmlElement parent,XmlElement node,String arguments)
		{
			Vector querySet = new Vector();
			int index = XmlHelper.getIndexOf(parent,node);
			querySet.addElement( new Integer(index) );

			return querySet;
		}
	}

	public XmlPathSearch(XmlElement document)
	{
		this.root = document;

		addProcessor( new XmlPathLastFunction() );
		addProcessor( new XmlPathPositionFunction() );
		addProcessor( new XmlMatchesFunction() );
	}

	public synchronized XmlElement findElement(String query)
	{
		return findElement(query,root);
	}

	public synchronized XmlElement findElement(String query,XmlElement node)
	{
		XmlDocument result = findElements(query,node);
		if(result.getElementCount()==1)
		{
			return result.getElementByIndex(0);
		}
		else
		{
			return null;
		}
	}

	public synchronized XmlDocument findElements(String query)
	{
		return findElements(query,root);
	}

	public synchronized XmlDocument findElements(String query,XmlElement node)
	{
		XmlDocument result = new XmlDocument("result-set");
		if(node==null)
		{
			return result;
		}		
		
		XmlElement startNode = node;

		if(query.length()==0)
		{
			startNode = root;
		}
		else if( query.charAt(0)=='/' )
		{
			query = query.substring(1);
			startNode = root;
		}

		String[] path = StringArrayHelper.parseFields(query,'/','[',']');
		int pathCursor = 0;

		doSearch( startNode,path,pathCursor,result );

		return result;
	}

	public void addProcessor(XmlPathFunction function)
	{
		processors.put(function.getName(),function);
	}

	public XmlElement getRoot()
	{
		return root;
	}
	
	private void doSearch( XmlElement startNode,String[] path,int pathCursor,XmlDocument result)
	{

		String subQuery = "";
		if(pathCursor<path.length)
		{
			subQuery = path[pathCursor];
		}

		if(subQuery.length()==0) // Is Wildcard Search at any level
		{
			if(pathCursor-1 == path.length) // If last clause in path
			{
				// include this node
				result.addChild( startNode );
				return;
			}

			// do search for this node using pathCursor+1
			doSearch(startNode,path,pathCursor+1,result);
			// do search for child nodes using pathCursor

			for(int loop=0;loop<startNode.getElementCount();loop++)
			{
				if(startNode.getElementByIndex(loop)!=null)
				{
					doSearch(startNode.getElementByIndex(loop),path,pathCursor,result);
				}
			}
			return;
		}

		String outerQuery = StringParser.getOutside(subQuery,'[',']');
		String innerQuery = StringParser.getInside(subQuery,'[',']');

		String nodeName = null;
		
		if(innerQuery!=null)
		{
			nodeName = outerQuery.substring(0,outerQuery.indexOf('['));
		}
		else
		{
			nodeName = outerQuery;
		}
		
		if(nodeName.equals("*"))
		{
			pathCursor--;
		}

		String matchingValue = null;
		if(nodeName.indexOf('=')>-1) //separate into nodeName and value for case (node=value)
		{
			matchingValue = nodeName.substring( nodeName.indexOf('=')+1 );
			nodeName = nodeName.substring( 0,nodeName.indexOf('=') );
		}
		
		if(
			!(startNode.getName().equals(nodeName) || nodeName.equals("node()")) &&	// Not any node or matching node 
			!nodeName.equals("*") 																// and Not Wildcard Search
		)
		{
			return;
		}

		if(matchingValue!=null)
		{
			if( passesQuery("\""+startNode.getValue()+"\"=\""+matchingValue+"\"",startNode)==false )
			{
				return;
			}
		}
		
		if(innerQuery!=null)
		{
			if(innerQuery.charAt(0)=='@')
			{
				if(passesQuery(innerQuery,startNode)==false)
				{
					return;
				}
			}
			else if( Character.isDigit(innerQuery.charAt(0)) )
			{
				int index = Integer.parseInt(innerQuery)-1;
				XmlElement parent = XmlHelper.getParent(root,startNode);
				if( parent==null || !(startNode.equals(parent.getElementByIndex(index))) )
				{
					return;
				}
			}
			else
			{
				boolean found = false;
				for(int loop=0;loop<startNode.getElementCount();loop++)
				{
					XmlElement nextNode = startNode.getElementByIndex(loop);
					
					if( nextNode!=null && findElements(innerQuery,nextNode).getElementCount()>0 )
					{
						found=true;
						break;
					}
					else if(nextNode==null)
					{
						if( process( innerQuery,startNode ).size()>0 )
						{
							found=true;
							break;
						}
					}
				}
				if(found==false)
				{
					return;
				}
			}
		}

		pathCursor++;
		if( pathCursor >= path.length)
		{
			result.addChild( startNode );
			return;
		}

		if( subQuery.equals("*") )
		{
			if( pathCursor==path.length )
			{
				result.addChild( startNode );
			}
			else
			{
				doSearch(startNode,path,pathCursor+1,result);
			}
		}
		
		if( startNode.getElementCount()==0 )
		{
			return;
		}

		for(int loop=0;loop<startNode.getElementCount();loop++)
		{
			if(startNode.getElementByIndex(loop)!=null)
			{
				doSearch(startNode.getElementByIndex(loop),path,pathCursor,result);
			}
		}

	}

	private boolean passesQuery(String query,XmlElement node)
	{
		String partA = query;
		String partB = null;
		String action="";

		if(query.indexOf('=')!=-1)
		{
			partA = query.substring(0,query.indexOf('='));
			partB = query.substring(query.indexOf('=')+1);
			action = "=";
		}
		else if(query.indexOf('>')!=-1)
		{
			partA = query.substring(0,query.indexOf('>'));
			partB = query.substring(query.indexOf('>')+1);
			action = ">";
		}
		else if(query.indexOf('<')!=-1)
		{
			partA = query.substring(0,query.indexOf('<'));
			partB = query.substring(query.indexOf('<')+1);
			action = "<";
		}

		Vector partsA = process(partA,node);
		Vector partsB = process(partB,node);

		if(partsB.size()==0)
		{
			if(partsA.size()==0)
			{
				return false;
			}
			else if(partsA.elementAt(0) instanceof Integer)
			{
				int requiredIndex = ((Integer)partsA.elementAt(0)).intValue()-1;

				XmlElement parent = XmlHelper.getParent(root,node);
				int index = XmlHelper.getIndexOf(parent,node);

				if(index == requiredIndex)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return true;
			}
		}

		for(int loopA=0;loopA<partsA.size();loopA++)
		{
			for(int loopB=0;loopB<partsB.size();loopB++)
			{
				Object objectA = partsA.elementAt(loopA);
				Object objectB = partsB.elementAt(loopA);

				if( isTrue(action,objectA,objectB) )
				{
						return true;
				}
			}
		}

		return false;
	}

	private Vector process(String part,XmlElement node)
	{
		Vector querySet = new Vector();

		if(part==null || part.length()==0)
		{
			return querySet;
		}

		if(part.charAt(0)=='@')
		{
			part = part.substring(1);
			if(part.equals("*"))
			{
				for(int loop=0;loop<node.getAttributeCount();loop++)
				{
					querySet.addElement( node.getAttributeValueByIndex(loop) );
				}
			}
			else
			{
				part = node.getAttributeValueByName(part);
				if(part!=null)
				{
					querySet.addElement( part );
				}
			}
		}
		else if(part.charAt(0)=='"' || part.charAt(0)=='\'')
		{
			part = part.substring(1,part.length()-1);
			querySet.addElement( part );
		}
		else if( Character.isDigit(part.charAt(0)) )
		{
			try{
				querySet.addElement( new Integer(part) );
			}
			catch(Exception e)
			{
				try{
					querySet.addElement( new BigDecimal(part) );
				}
				catch(Exception e2)
				{
				}
			}
		}
		else if(part.lastIndexOf(')')==part.length()-1)
		{
			String arguments = StringParser.getInside(part,'(',')');
			String functionName = StringParser.getOutside(part,'(',')');

			querySet.addAll( processFunction(functionName,arguments,node) );
		}
		else
		{
			node = node.getElementByName(part);
			if(node==null)
			{
				return querySet;
			}
			querySet.addElement( node.getValue() );
		}

		return querySet;
	}

	private Vector processFunction(String functionName,String arguments,XmlElement node)
	{
		if(functionName.indexOf('(')!=-1)
		{
			functionName = functionName.substring(0,functionName.indexOf('('));
		}
		
		Vector querySet = new Vector();
		XmlElement parent = XmlHelper.getParent(root,node);

		XmlPathFunction function = (XmlPathFunction)processors.get(functionName);

		if(function!=null)
		{
			return function.process(parent,node,arguments);
		}

		return querySet;
	}

	private boolean isTrue(String action,Object a,Object b)
	{
		String result="?";

		if(b instanceof String)
		{
			if( !(a instanceof String) )
			{
				a = ""+a;
			}

			result = compare((String)a,(String)b);
		}
		else if(b instanceof Integer)
		{
			if( !(a instanceof Integer) )
			{
				try{
					a = new Integer(""+a);
				}
				catch(Exception e)
				{
					return false;
				}
			}

			result = compare((Integer)a,(Integer)b);
		}
		else if(b instanceof BigDecimal)
		{
			if( !(a instanceof BigDecimal) )
			{
				try{
					a = new BigDecimal(""+a);
				}
				catch(Exception e)
				{
					return false;
				}
			}

			result = compare((BigDecimal)a,(BigDecimal)b);
		}

		if(result.equals(action))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	private String compare(String a,String b)
	{
		if(a.equals(b))
		{
			return "=";
		}
		return "!=";
	}

	private String compare(Integer a,Integer b)
	{
		switch( a.compareTo(b) )
		{
			case 0:
				return "=";
			case 1:
				return ">";
			case -1:
				return "<";
			default:
				return "?";
		}
	}

	private String compare(BigDecimal a,BigDecimal b)
	{
		switch( a.compareTo(b) )
		{
			case 0:
				return "=";
			case 1:
				return ">";
			case -1:
				return "<";
			default:
				return "?";
		}
	}
}
